package im.zego.zpns_flutter.internal;

import android.content.Context;
import android.content.Intent;
import android.content.pm.ApplicationInfo;
import android.content.pm.PackageManager;
import android.os.Handler;
import android.os.Looper;
import android.util.Log;

import im.zego.zpns_flutter.callback.ZPNsBackgroundMessageHandler;
import im.zego.zpns_flutter.internal.utils.ContextHolder;
import im.zego.zpns_flutter.internal.utils.ZPNsLogWriter;
import im.zego.zpns_flutter.internal.utils.ZPNsMessagingBackgroundService;
import im.zego.zpns_flutter.internal.utils.ParcelableMap;
import im.zego.zpns_flutter.internal.utils.ZPNsMessagingUtils;

import java.lang.reflect.Method;
import java.util.HashMap;

import im.zego.zpns.ZPNsMessageReceiver;
import im.zego.zpns.entity.ZPNsMessage;
import im.zego.zpns.entity.ZPNsRegisterMessage;

import im.zego.zpns.enums.ZPNsConstants;
import io.flutter.plugin.common.EventChannel;

public class ZPNsPluginEventHandler extends ZPNsMessageReceiver {

    public static EventChannel.EventSink mySink = null;

    public void setSink(EventChannel.EventSink sink) {
        mySink = sink;
    }

    private static final String CHANNEL = "widget.filc.hu/timetable";

    private static final String TAG = "ZPNsPluginEventHandler";

    @Override
    protected void onThroughMessageReceived(Context context, ZPNsMessage message) {
        Handler handler = new Handler(Looper.getMainLooper());
        if (ContextHolder.getApplicationContext() == null) {
            ContextHolder.setApplicationContext(context.getApplicationContext());
        }
        HashMap<String, Object> resultMap = new HashMap<>();
        resultMap.put("method", "onThroughMessageReceived");
        resultMap.put("message", ZPNsConverter.cnvZPNsMessageObjectToMap(message));

        if (ZPNsMessagingUtils.isApplicationForeground(context)) {
            handler.post(() -> {
                if (mySink == null) {
                    return;
                }
                mySink.success(resultMap);
            });
            return;
        }
        //  |-> ---------------------
        //    App in Background/Quit
        //   ------------------------
        boolean useWakefulService = getUseWakeFulService(message);
        Intent onBackgroundMessageIntent =
                new Intent(context, ZPNsMessagingBackgroundService.class);
        ParcelableMap parcelableMap = new ParcelableMap(resultMap);
        onBackgroundMessageIntent.putExtra(
                ZPNsMessagingUtils.EXTRA_REMOTE_MESSAGE, parcelableMap);
        ZPNsMessagingBackgroundService.enqueueMessageProcessing(
                context, onBackgroundMessageIntent, useWakefulService);


        try {
            ApplicationInfo appInfo = context.getPackageManager().getApplicationInfo(context.getPackageName(),
                    PackageManager.GET_META_DATA);
            String classPath = appInfo.metaData.getString("ZPNsBackgroundMessageHandlerClassPath");
            if (classPath != null) {
                try {
                    Class<?> zpnsHandlerClass = Class.forName(classPath);
                    ZPNsBackgroundMessageHandler zpnsHandlerObj = (ZPNsBackgroundMessageHandler) zpnsHandlerClass.newInstance();
                    zpnsHandlerObj.onThroughMessageReceived(context, message);
                } catch (Exception | Error e) {
                    Log.d(TAG, "ZPNs Native Find class failure", e);
                }
            }
        } catch (Exception | Error e) {
            Log.d(TAG, "ZPNs Native Find ApplicationInfo failure", e);
        }

//        handler.post(() -> {
//            Log.d("ZPNs native", "onThroughMessageReceived post start");
//            FlutterLoader loader = new FlutterLoader();
//            loader.startInitialization(context);
//            loader.ensureInitializationComplete(context,new String[0]);
//            FlutterEngine engine = new FlutterEngine(context.getApplicationContext());
//            DartExecutor.DartEntrypoint entrypoint = new DartExecutor.DartEntrypoint("lib/main.dart", "goodJob");
//            engine.getDartExecutor().executeDartEntrypoint(entrypoint);
//            new MethodChannel(engine.getDartExecutor().getBinaryMessenger(), CHANNEL);
//            Log.d("ZPNs native", "onThroughMessageReceived post finish");
//        });
//        if(mySink == null){
//            Log.d("ZPNs native", "onThroughMessageReceived my sink is null");
//            return;
//        }

        //Log.d("ZPNs native", "onThroughMessageReceived my sink is not null");
        //handler.post(() -> mySink.success(resultMap));
    }

    private boolean getUseWakeFulService(ZPNsMessage message) {
        if (message.getPushSource() == ZPNsConstants.PushSource.FCM) {
            try {
                Object fcmMessage = message.getPushMessage();
                Class<?> classz = fcmMessage.getClass();
                Method method = classz.getDeclaredMethod("getPriority");
                method.setAccessible(true);
                return ((int) method.invoke(fcmMessage) == 1);
            } catch (Exception | Error ignored) {
            }
        }
        return false;
    }

    @Override
    protected void onNotificationClicked(Context context, ZPNsMessage message) {
        if (mySink == null) {
            return;
        }
        HashMap<String, Object> resultMap = new HashMap<>();
        resultMap.put("method", "onNotificationClicked");
        resultMap.put("message", ZPNsConverter.cnvZPNsMessageObjectToMap(message));

        Handler handler = new Handler(Looper.getMainLooper());
        handler.post(() -> mySink.success(resultMap));
    }

    @Override
    protected void onNotificationArrived(Context context, ZPNsMessage message) {
        if (mySink == null) {
            return;
        }
        HashMap<String, Object> resultMap = new HashMap<>();
        resultMap.put("message", ZPNsConverter.cnvZPNsMessageObjectToMap(message));
        resultMap.put("method", "onNotificationArrived");

        Handler handler = new Handler(Looper.getMainLooper());
        handler.post(() -> mySink.success(resultMap));
    }

    @Override
    protected void onRegistered(Context context, ZPNsRegisterMessage message) {
        Log.d("ZPNs native", "onRegistered: pushid:" + message.getCommandResult());
        if (mySink == null) {
            return;
        }
        HashMap<String, Object> resultMap = new HashMap<>();

        resultMap.put("method", "onRegistered");

        resultMap.put("message", ZPNsConverter.cnvZPNsRegisterMessageObjectToMap(message));
        Handler handler = new Handler(Looper.getMainLooper());
        handler.post(() -> mySink.success(resultMap));
        ZPNsLogWriter.writeLog("onRegistered,code:"+message.getErrorCode()+" message:"+message.getMsg()+" commandResult:"+message.getCommandResult()+" deveiceToken:"+message.getDeviceToken());
    }
}
