#import "ZegoZpnsPlugin.h"
#import <ZPNs/ZPNs.h>
#import "internal/utils/NSDictionary+safeInvoke.h"
#import "PushKit/PushKit.h"
#import "CallKitManager.h"
#import "ZPNsSettings.h"
#import "ZPNsEventManager.h"
#import "ZPNsMethodManager.h"
#import "CallKit/CallKit.h"
#import "internal/utils/NSMutableDictionary+safeInvoke.h"
#import "internal/utils/ZPNsUtils.h"
@interface ZegoZpnsPlugin ()<ZPNsNotificationCenterDelegate>

@property FlutterMethodChannel *methodChannel;

@property (nonatomic, strong) FlutterEventChannel *eventChannel;



@end

@implementation ZegoZpnsPlugin
+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
    FlutterMethodChannel *methodChannel = [FlutterMethodChannel
      methodChannelWithName:@"zego_zpns"
            binaryMessenger:[registrar messenger]];
  ZegoZpnsPlugin* instance = [[ZegoZpnsPlugin alloc] init];
  [registrar addApplicationDelegate:instance];
  [registrar addMethodCallDelegate:instance channel:methodChannel];
  UNUserNotificationCenter *center = [UNUserNotificationCenter currentNotificationCenter];
  center.delegate = (id)[ZPNs shared];
  [[ZPNs shared] setZPNsNotificationCenterDelegate:(id)[ZPNsEventManager sharedInstance]];
  FlutterEventChannel *eventChannel = [FlutterEventChannel eventChannelWithName:@"zpns_event_handler" binaryMessenger:[registrar messenger]];
  [eventChannel setStreamHandler:(id)instance];
  instance.eventChannel = eventChannel;
}

- (FlutterError* _Nullable)onListenWithArguments:(id _Nullable)arguments
                                       eventSink:(FlutterEventSink)events {
    [ZPNsSettings setEvent:events];
    return nil;
}

- (FlutterError* _Nullable)onCancelWithArguments:(id _Nullable)arguments {
    return nil;
}

- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
    SEL selector = NSSelectorFromString([NSString stringWithFormat:@"%@:result:", call.method]);
    
    if (![[ZPNsMethodManager sharedInstance] respondsToSelector:selector]) {
        result(FlutterMethodNotImplemented);
        return;
    }
    NSMethodSignature *signature = [[ZPNsMethodManager sharedInstance] methodSignatureForSelector:selector];

    NSInvocation* invocation = [NSInvocation invocationWithMethodSignature:signature];

    invocation.target = [ZPNsMethodManager sharedInstance];
    invocation.selector = selector;

    [invocation setArgument:&call atIndex:2];
    [invocation setArgument:&result atIndex:3];

    [invocation invoke];
}
#pragma mark - AppDelegate

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions {
    return YES;
}

- (void)application:(UIApplication *)application didRegisterForRemoteNotificationsWithDeviceToken:(NSData *)deviceToken {
    [[ZPNs shared] setDeviceToken:deviceToken isProduct:!([ZPNsMethodManager sharedInstance].debug)];
    [ZPNsUtils wirteCustomLog:[NSString stringWithFormat:@"didRegisterForRemoteNotificationsWithDeviceToken,set deveice token,device token:%@,isProduct:%@",[ZPNsUtils getHexStringForData:deviceToken],!([ZPNsMethodManager sharedInstance].debug)?@"true":@"false"]];
}

- (void)application:(UIApplication*)application
didFailToRegisterForRemoteNotificationsWithError:(NSError*)error{
    if(ZPNsSettings.event == nil){
        return;
    }
    NSDictionary *resultDic = @{@"method":@"onRegistered",@"message":@{@"pushID":@"",@"errorCode":[NSNumber numberWithInteger:error.code],@"pushSourceType":[NSNumber numberWithInt:10],@"msg":error.localizedDescription?error.localizedDescription:@"",@"commandResult":@""}};
    [ZPNsUtils wirteCustomLog:[NSString stringWithFormat:@"didFailToRegisterForRemoteNotificationsWithError,error code:%ld,error message:%@",(long)error.code,error.localizedDescription?error.localizedDescription:@""]];
    ZPNsSettings.event(resultDic);
}

- (BOOL)application:(UIApplication *)application didReceiveRemoteNotification:(NSDictionary *)userInfo fetchCompletionHandler:(void (^)(UIBackgroundFetchResult result))completionHandler API_AVAILABLE(ios(7.0)){
    if(ZPNsSettings.event == nil){
        return true;
    }
    NSString *completionSeq = [ZPNsSettings generateSeqStr];
    [ZPNsSettings.didReceiveRemoteNotificationCompletionSeqMap safeSetObject:completionHandler forKey:completionSeq];
    
    NSDictionary *resultDic = @{@"method":@"onThroughMessageReceived",@"userInfo":userInfo,@"seq":completionSeq};
    ZPNsSettings.event(resultDic);
    return true;
}
@end
