
import 'dart:io';
import 'package:zego_zpns/src/internal/call_kit_engine.dart';
import 'package:zego_zpns/src/internal/zpns_converter.dart';
import 'package:zego_zpns/src/internal/zpns_defines_extension.dart';
import 'package:zego_zpns/src/internal/zpns_settings.dart';
import 'package:zego_zpns/src/ios_utils/internal/uuid_impl.dart';
import 'package:zego_zpns/zego_zpns.dart';

class ZPNsEventHandlerImpl implements ZPNsEventHandler {

  static void registerEventHandler() async {
    ZPNsSettings.streamSubscription ??=
        ZPNsSettings.eventChannel.receiveBroadcastStream().listen(eventListener);

  }

  static void unregisterEventHandler() async {
    await ZPNsSettings.streamSubscription?.cancel();
    ZPNsSettings.streamSubscription = null;
  }

  static void eventListener(dynamic data) {
    final Map<dynamic, dynamic> map = data;
    switch (map['method']) {
      case 'onRegistered':
        if (ZPNsEventHandler.onRegistered == null) return;
        ZPNsRegisterMessage registerMessage =
            ZPNsConverter.cnvZPNsRegisterMessageMapToObject(map['message']);
        ZPNsEventHandler.onRegistered!(registerMessage);
        break;
      case 'onNotificationArrived':
        if (ZPNsEventHandler.onNotificationArrived == null) return;
        ZPNsMessage message =
            ZPNsConverter.cnvZPNsMessageMapToObject(map['message']);
        ZPNsEventHandler.onNotificationArrived!(message);
        break;
      case 'onNotificationClicked':
        if (ZPNsEventHandler.onNotificationClicked == null) return;

        ZPNsMessage message =
            ZPNsConverter.cnvZPNsMessageMapToObject(map['message']);

        ZPNsEventHandler.onNotificationClicked!(message);
        break;
      case 'onThroughMessageReceived':
      if (ZPNsEventHandler.onThroughMessageReceived == null) return;
        if(Platform.isIOS){
            ZPNsMessage message = ZPNsMessage(pushSourceType: ZPNsPushSourceType.APNs);
            message.extras = map['userInfo'];
            String seq = map['seq'];
            ZPNsEventHandler.onThroughMessageReceived!(message,iOSOnThroughMessageReceivedCompletion = (ZPNsIOSBackgroundFetchResult result){
              if(Platform.isIOS){
                ZPNsSettings.channel.invokeMethod('didReceiveRemoteNotificationCompletion',{"seq":seq,"result":ZPNsIOSBackgroundFetchResultExtension.valueMap[result]});
              }
            });
        }else if(Platform.isAndroid){
          ZPNsMessage message = ZPNsConverter.cnvZPNsMessageMapToObject(map['message']);
          ZPNsEventHandler.onThroughMessageReceived!(message,null);
        }
        break;
      case 'didReceiveIncomingPushWithPayload':
        if(CallKitEventHandler.didReceiveIncomingPush == null)return;
        String uuidSeq = map['uuidSeq'];
        UUIDImpl uuid = UUIDImpl();
        uuid.seq = uuidSeq;
        CallKitEventHandler.didReceiveIncomingPush!(map['payload']??{},uuid);
        break;
      case 'providerDidReset':
        if(CallKitEventHandler.providerDidReset == null) return;
        CallKitEventHandler.providerDidReset!();
        break;
      case 'providerDidBegin':
        if(CallKitEventHandler.providerDidBegin == null) return;
        CallKitEventHandler.providerDidBegin!();
        break;
      case 'didActivateAudioSession':
        if(CallKitEventHandler.didActivateAudioSession == null) return;
        CallKitEventHandler.didActivateAudioSession!();
        break;
      case 'didDeactivateAudioSession':
        if(CallKitEventHandler.didDeactivateAudioSession == null) return;
        CallKitEventHandler.didDeactivateAudioSession!();
        break;
      case 'timedOutPerformingAction':
        if(CallKitEventHandler.timedOutPerformingAction == null) return;
        CallKitEventHandler.timedOutPerformingAction!(CXActionImpl(seq: map['seq']));
        break;
      case 'executeTransaction':
        // if(CallKitEventHandler.executeTransaction == null) return;
        // CallKitEventHandler.executeTransaction!(CXActionImpl(seq: map['seq']));
        break;
      case 'performStartCallAction':
        if(CallKitEventHandler.performStartCallAction == null) return;
        CallKitEventHandler.performStartCallAction!(CXActionImpl(seq: map['seq']));
        break;
      case 'performAnswerCallAction':
        if(CallKitEventHandler.performAnswerCallAction == null) return;
        CallKitEventHandler.performAnswerCallAction!(CXActionImpl(seq: map['seq']));
        break;
      case 'performEndCallAction':
        if(CallKitEventHandler.performEndCallAction == null) return;
        CallKitEventHandler.performEndCallAction!(CXActionImpl(seq: map['seq']));
        break;
      case 'performSetHeldCallAction':
        if(CallKitEventHandler.performSetHeldCallAction == null) return;
        CallKitEventHandler.performSetHeldCallAction!(CXActionImpl(seq: map['seq']));
        break;
      case 'performSetMutedCallAction':
        if(CallKitEventHandler.performSetMutedCallAction == null) return;
        CallKitEventHandler.performSetMutedCallAction!(CXSetMutedCallActionImpl(muted: map['muted'],seq: map['seq']));
        break;
      case 'performSetGroupCallAction':
        if(CallKitEventHandler.performSetGroupCallAction == null) return;
        CallKitEventHandler.performSetGroupCallAction!(CXActionImpl(seq: map['seq']));
        break;
      case 'performPlayDTMFCallAction':
        if(CallKitEventHandler.performPlayDTMFCallAction == null) return;
        CallKitEventHandler.performPlayDTMFCallAction!(CXActionImpl(seq: map['seq']));
        break;
      default:
    }

  }


}
